/*******************************************************************************\
 *			SeaBattle - D.Bernaudeau, J. Vehent - 2006 	       *
 *									       *
 *	fonctions jeu: implmentation de l'interface, des tours de jeu, des    *
 *		tirs et rponses, manipulation des matrices de jeu	       *
 *									       *
\*******************************************************************************/ 

/*! \file jeu.c
    \brief fonctions jeu

    implmentation de l'interface, des tours de jeu, des tirs et rponses, manipulation des matrices de jeu	

    \Author D.Bernaudeau
    \Author J. Vehent
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ncurses.h>

#include "affichage.h"
#include "test.h"
#include "reseau.h"

/*VARIABLES GLOBALES 
*/

/*!
  \var goflag
  \brief 1: joueur pret  jouer
 */
int goflag=0;

/*!
  \var advgoflag
  \brief 1: joueur adverse pret  jouer
 */
int advgoflag=0;

/*!
  \var flagjeu
  \brief 1: le joueur doit jouer
 */
int flagjeu=0;

/*!
  \var shoot_ligne
  \brief ligne du tir jou
 */
int shoot_ligne=0;

/*!
  \var shoot_colonne
  \brief colonne du tir jou
 */
int shoot_colonne=0;



/*!
  \var matrice_locale
  \brief matrice contenant les positions des bateaux du joueur

	MATRICES DES GRILLES DE JEU 
	les matrices sont remplient avec les infos suivantes:
	0: case vide, c'est de l'eau
	1: case vide touche par un tir
	4: porte avion
	5: croiseur 1
	6: croiseur 2
	7: croiseur 3
	8: corvette 1
	9: corvette 2

	les valeurs oppose des bateaux signifie qu'ils sont touchs

	quand on bateau est coul, ses cases ont la valeur opposes moins 10
	ex: porte avion coule: toutes ses cases ont la valeur -14
 */
int matrice_locale[10][10];

/*!
  \var matrice_adverse
  \brief matrice contenant les coups jous
 */
int matrice_adverse[10][10];


/*!
  \var bateaux_dispo
  \brief tableau contenant la disponibilit des bateaux

  sont disponibles: 1 porte-avion (n1), 3 croiseurs( n2  4) et 2 corvettes (n5 et 6)

 */
int bateaux_dispo[6]={1,1,1,1,1,1};

/* FIN VARIABLES GLOBALES
*/


/***************************************************************************************************************/

/// Initialise les cases des deux matrices  0
/*!
\return 0 si tout se passe bien
*/
int init_matrices(){
	int i,j;
	for(i=0;i<10;i++){
		for(j=0;j<10;j++){
			matrice_locale[i][j]=0;
			matrice_adverse[i][j]=0;
		}
	}
	return 0;
}


/***************************************************************************************************************/

//! Affiche une erreur lors de l'appel a une action interdite
/*!
\param[in] flag le code d'erreur a afficher
\return 0 si tout s'est bien pass
*/
int interdit(int flag){
	switch(flag){
		case 1: afficher_erreur("Dbut du jeu activ, les actions set et unset sont interdites");break;
		case 2: afficher_erreur("Vous etes encore en prparation du jeu, shoot interdit");break;
		case 3: afficher_erreur("Vous avez dj dbut le jeu, action go interdite");break;
		case 4: afficher_erreur("Vous n'avez pas plac tout vos bateaux, action go interdite");break;
		case 5: afficher_erreur("Commande inconnue ou mal formule");break;
		case 6: afficher_erreur("Vous avez dj jou ce coup ! Jouez un autre coup.");break;
		case 7: afficher_erreur("Ce n'est pas  vous de jouer...");break;

		default: return -1;
	}
	return 0;
}


/***************************************************************************************************************/

//! fonction de placement des bateaux sur la grille locale
/*!
\param[in] msg la chaine de caractres contenant les paramtres de placement
\return 0 si tout s'est bien pass, -1 en cas d'erreur
*/
int set(char *msg){

	// On stocke les informations de positions dans les variables ci-dessous
	char colonnec=msg[7]; int colonne;
	char typec=msg[5];int type, longueur;
	char lignec=msg[9];int ligne;
	int orientationc = msg[11];int orientation;

	//iterateur
	int i;
	
	//les conversions
	switch(colonnec){
		case 'A': colonne = 0;break;
		case 'B': colonne = 1;break;
		case 'C': colonne = 2;break;
		case 'D': colonne = 3;break;
		case 'E': colonne = 4;break;
		case 'F': colonne = 5;break;
		case 'G': colonne = 6;break;
		case 'H': colonne = 7;break;
		case 'I': colonne = 8;break;
		case 'J': colonne = 9;break;
		default: interdit(5);return -1;
	}

	switch(typec){
		case '1': type=1;break;
		case '2': type=2;break;
		case '3': type=3;break;
		case '4': type=4;break;
		case '5': type=5;break;
		case '6': type=6;break;
		default: interdit(5);return -1;
	}
	if (type==1)longueur=5;
	else{ if (type>4)longueur=2;else longueur=3;}
	
	switch(lignec){
		case '0': ligne = 0;break;
		case '1': ligne = 1;break;
		case '2': ligne = 2;break;
		case '3': ligne = 3;break;
		case '4': ligne = 4;break;
		case '5': ligne = 5;break;
		case '6': ligne = 6;break;
		case '7': ligne = 7;break;
		case '8': ligne = 8;break;
		case '9': ligne = 9;break;
		default: interdit(5);return -1;
	}	

	switch(orientationc){
		case 'H': orientation=0;break;
		case 'V': orientation=1;break;
		default: interdit(5);return -1;
	}

	/* CONTROLES D'INTEGRITE:
	 * 1. vrification que le bateau tiens entierement dans la grille
	 * 2. vrification que le bateau ne chevauche pas un autre bateau dj en place
	 */
	if(sortie_matrice(longueur,colonne,ligne,orientation)!=0){

		afficher_erreur("Impossible de placer le bateau  l'emplacement demand pour cause de sortie de territoire");
		return -2;
	}else if(set_collision(matrice_locale,longueur,colonne,ligne,orientation)<0){
 
		afficher_erreur("Impossible de placer le bateau  l'emplacement demand pour cause de collision");
		return -2;
	}

		
	//on test la disponibilit du bateau, si il est dispo on le place dans la matrice
	//on numrote a partir de 0 donc il faut soustraire 1  la valeur de type
	if (0 > (bateaux_dispo[type-1]-1)) {
		afficher_erreur("Erreur ! bateau non disponible...");
	} else {
		//On stocke ces informations dans la matrice, 2 est le numro des bateaux intacts
		matrice_locale[ligne][colonne]=type+3;
		
		//on dcrmente la disponibilit
		//on numrote a partir de 0 donc il faut soustraire 1  la valeur de type
		--bateaux_dispo[type-1];
		
		//et on place le bateau dans la matrice selon son orientation
		//dans la matrice on numrote les type a partir de 4 donc il faut ajouter 3 a la valeur de type
		if(orientation==0){
			for(i=1;(colonne+i)<(colonne+longueur); i++) matrice_locale[ligne][colonne+i]=type+3;
		}else{
			if (orientation==1){
				for(i=1;(ligne+i)<(ligne+longueur); i++) matrice_locale[ligne+i][colonne]=type+3;
			}else return -1;
		}
	}

	//rafraichissement de l'affichage
	effacer_bateau(type);
	afficher_matrice(1, matrice_locale);
	
	return 0;
}


/***************************************************************************************************************/

//! fonction permettant d'enlever de la grille un bateau plac
/*!
\param msg la chaine de caractres contenant les paramtres du bateau  dplacer
\return 0
*/
int unset(char *msg){
	afficher_erreur("cette fonctionnalit n'est pas encore disponible");
	return 0;
}


/***************************************************************************************************************/

//! Dmarrer la partie quand les deux joueurs sont pret
void demarre_jeu(){
	afficher_erreur("LES DEUX CAMPS SONT PRET.... DEMARRAGE DU JEU");
	afficher_etat(flagjeu);
}


/***************************************************************************************************************/

//! Recupre l'etat de l'adversaire et dmarre le jeu si le joueur est pret
void adv_go(){
	advgoflag=1;
	if (goflag==1) demarre_jeu();

}

/***************************************************************************************************************/

//! Affichage l'aide pour le placement des bateaux
/*!
\return 0 si tout se passe bien
*/
int help(){
	move(28,1);clrtoeol();
	mvprintw(28,(COLS-67)/2,"Commande SET '/set' = PLACEMENT DES BATEAUX SUR LA GRILLE DU JOUEUR");
	move(29,1);clrtoeol();
	mvprintw(29,(COLS-77)/2, "utilisation: /set [TYPE BATEAU] [LETTRE COLONNE] [NUMERO LIGNE] [ORIENTATION]");
	move(30,1);clrtoeol();
	mvprintw(30,(COLS-104)/2,"TYPE BATEAU: numro de gauche dans la colonne centrale   |   ORIENTATION: H pour horizontal, V pour vertical");
	move(31,1);clrtoeol();
	mvprintw(32,(COLS-102)/2,"exemples: /set 1 B 3 H place le porte avion de B3 a F3  |   /set 3 E 4 V place le croiseur n2 de E4 a E6");
	
	refresh();
	return 0;
}


/***************************************************************************************************************/

//! Tirer sur une case de l'adversaire
/*!
\param[in] msg la chaine de caractres contenant les coordonnes du tir
\return 0 si tout se passe bien
*/
/* on rcupre la colonne et la ligne du tir et on effectue un controle sur
 * matrice_adverse pour voir si ca n'a pas dj t jou
 * 
 * si le controle est ngatif, on envoie le tir a l'adversaire par le rseau
*/
int shoot(char *msg){
	char colonnec=msg[7];
	char lignec=msg[9];

	//les conversions
	//on utilise des variables globales car elles sont rappels dans rep_shoot
	switch(colonnec){
		case 'A': shoot_colonne = 0;break;
		case 'B': shoot_colonne = 1;break;
		case 'C': shoot_colonne = 2;break;
		case 'D': shoot_colonne = 3;break;
		case 'E': shoot_colonne = 4;break;
		case 'F': shoot_colonne = 5;break;
		case 'G': shoot_colonne = 6;break;
		case 'H': shoot_colonne = 7;break;
		case 'I': shoot_colonne = 8;break;
		case 'J': shoot_colonne = 9;break;
		default: interdit(5);return -1;
	}
	switch(lignec){
		case '0': shoot_ligne = 0;break;
		case '1': shoot_ligne = 1;break;
		case '2': shoot_ligne = 2;break;
		case '3': shoot_ligne = 3;break;
		case '4': shoot_ligne = 4;break;
		case '5': shoot_ligne = 5;break;
		case '6': shoot_ligne = 6;break;
		case '7': shoot_ligne = 7;break;
		case '8': shoot_ligne = 8;break;
		case '9': shoot_ligne = 9;break;
		default: interdit(5);return -1;
	}
	
	//si la case n'a pas dj t joue, on envoie le coup a l'adversaire
	if(matrice_adverse[shoot_ligne][shoot_colonne] > -1 && matrice_adverse[shoot_ligne][shoot_colonne]!=1){
		send_shoot(msg);
	//sinon on refuse le coup
	}else interdit(6);
	return 0;
}


/***************************************************************************************************************/

//! Traitement des tirs envoys par l'adversaire
/*!
\param[in] msg la chaine de caractres contenant les coordonnes du tir
*/
/* on modifie l'tat de la matrice au coordonnes saisie
 * et on appelle la fonction de mise  jour de l'affichage
*/
void recv_shoot(char *msg){
	char colonnec=msg[7];
	char lignec=msg[9];
	
	int colonne,ligne, i;

	//les conversions
	switch(colonnec){
		case 'A': colonne = 0;break;
		case 'B': colonne = 1;break;
		case 'C': colonne = 2;break;
		case 'D': colonne = 3;break;
		case 'E': colonne = 4;break;
		case 'F': colonne = 5;break;
		case 'G': colonne = 6;break;
		case 'H': colonne = 7;break;
		case 'I': colonne = 8;break;
		case 'J': colonne = 9;break;
		default: break;
	}
	switch(lignec){
		case '0': ligne = 0;break;
		case '1': ligne = 1;break;
		case '2': ligne = 2;break;
		case '3': ligne = 3;break;
		case '4': ligne = 4;break;
		case '5': ligne = 5;break;
		case '6': ligne = 6;break;
		case '7': ligne = 7;break;
		case '8': ligne = 8;break;
		case '9': ligne = 9;break;
		default: break;
	}

	//on test les coups dans l'eau
	if (matrice_locale[ligne][colonne]==0) matrice_locale[ligne][colonne]=1;

	//on test les bateaux touchs, si un bateau est touch, son chiffre est oppos
	else {	if(matrice_locale[ligne][colonne]>3){
			matrice_locale[ligne][colonne] = matrice_locale[ligne][colonne]*(-1);
		
			//on verifie s'il ne reste plus de cases intactes pour ce bateau 
			if( 1 ==  couler(matrice_locale, ligne, colonne) )
			{
				/// Couler le bateau
				/// une case touche a un id ngatif, si le bateau est coul on soustrait 10 a cet id

				// Bateau horizontal
				for(i=0;i<10;i++){
					if(	(matrice_locale[ligne][i] == matrice_locale[ligne][colonne]) 
						&& 
						(i != colonne) 
					)
						matrice_locale[ligne][i] = (matrice_locale[ligne][i] - 10);
				}
				// Bateau vertical
				for(i=0;i<10;i++){
					if(	(matrice_locale[i][colonne]== matrice_locale[ligne][colonne])
						&& 
						(i != ligne)
					)
						matrice_locale[i][colonne] = (matrice_locale[i][colonne] - 10);
				}
				
				//on a coul les autres cases, on coule enfin la case vise
				matrice_locale[ligne][colonne] = (matrice_locale[ligne][colonne] - 10);
	
				//si on a perdu, la partie est finie
				if( 1 == perdu(matrice_locale)){
					flagjeu=-10;
					afficher_erreur("DOMMAGE !!! VOUS AVEZ PERDU LA PARTIE !!!");
				}
			}
		}
	}
	//on envoie la rponse  l'adversaire par le rseau 
	send_rep_shoot(matrice_locale[ligne][colonne]);

	//et on raffraichit l'affichage
	afficher_matrice(1,matrice_locale);

	//enfin donne la main au joueur
	if (flagjeu != -10) flagjeu=1;
	afficher_etat(flagjeu);
}


/***************************************************************************************************************/

//! Analyse la rponse renvoy suite  un tir
/*!
\param[in] msg la chaine de caractres contenant la rponse au tir
*/
void rep_shoot(char *msg){
	int reponse = atoi(msg), i;

	/* on test le code de retour et on dite la grille adverse en fonction
	 */
	switch(reponse){
		case 1:	afficher_erreur("rat ! un coup dans l'eau");
			matrice_adverse[shoot_ligne][shoot_colonne]=1;
			break;
		
		case -4 : case -5 : case -6 : case -7 : case -8 : case -9 : 
			afficher_erreur("vous avez touch un bateau adverse !"); 
			matrice_adverse[shoot_ligne][shoot_colonne] = reponse;
			break;
	
			/// si un bateau est coul, on vrifie que la partie n'est pas termine
		case -14 : case -15 : case -16 : case -17 : case -18 : case -19 : 
			afficher_erreur("vous avez coul un bateau adverse !"); 
			matrice_adverse[shoot_ligne][shoot_colonne]=reponse;
			
			/// Couler le bateau
			/// une case touche a un id ngatif, si le bateau est coul on soustrait 10 a cet id
			// Bateau horizontal
			for(i=0;i<10;i++){
				if(	(matrice_adverse[shoot_ligne][i] == (reponse+10)) 
					&& 
					(i != shoot_colonne) 
				)
					matrice_adverse[shoot_ligne][i] = reponse;
			}
			// Bateau vertical
			for(i=0;i<10;i++){
				if(	(matrice_adverse[i][shoot_colonne]== (reponse+10))
					&& 
					(i != shoot_ligne)
				)
					matrice_adverse[i][shoot_colonne] = reponse;
			}
			
			if (1 == perdu(matrice_adverse)){
				afficher_erreur("BRAVO, VOUS AVEZ GAGNE LA PARTIE !!!!");
				flagjeu=10;
			}
			break;
			
		default : afficher_erreur("erreur lors de la lecture de la rponse");break;
	}	

	//on raffraichit l'affichage
	afficher_matrice(2, matrice_adverse);
	
	//on rend la main au joueur
	if( flagjeu != 10) flagjeu=0;
	afficher_etat(flagjeu);
}

//! Controler la chaine de caractre pour trouver l'action a effectuer
/*!
 \param[in] msg la chaine de caractre saisie par le joueur
 \return 0 si tout s'est bien pass, -1 en cas d'erreur
*/
int interpretation(char *msg){


	//definition des chaines d'actions
	char set_c[]="/set", unset_c[]="/unset", go_c[]="/go", help_c[]="/help", shoot_c[]="/shoot";

	//	TEST DE L'ACTION A EFFECTUER. on compare la chaine saisie avec les paterns prcdent 

	//test de /set
	if (0 == strncmp(msg, set_c,4)){ 
		if (goflag == 0){ if(-1 == set(msg)) return -1;} 
		else interdit(1);}

	else{ 	//test de "unset"
		if (0 == strncmp(msg, unset_c,6)){ 
			if (goflag==0) unset(msg); 
			else interdit(1);}

		else{ 	//test de "go" et dmarre le jeu si l'adversaire a dj lanc /go
			if (0 == strncmp(msg, go_c,3)){ 
				//on ne peut dmarrer que si tous les bateaux sont placs
				if (bateaux_dispo[2]==0 && bateaux_dispo[3]==0 && bateaux_dispo[5]==0){
					//celui qui passe /go en premier commence a jouer
					if (goflag == 0){ 
						goflag=1; 
						if(advgoflag==0){ 
							flagjeu=1;
							afficher_erreur("Vous tes pret a jouer, attente de l'adversaire");
							return 0;
						} else demarre_jeu(); 
					}else interdit(3);
				} else {interdit(4);return -2;}}
			else{ 	//test de "help"
				if (0 == strncmp(msg, help_c,5)){ help();}

				else{ 	//test de "shoot"
					if (0 == strncmp(msg, shoot_c,6)){ 
						//verification que c'est bien au joueur de jouer
						if (flagjeu == 1){
							//verification que l'on est en mode jeu
							if (goflag!=0 && advgoflag!=0) shoot(msg); 
							else interdit(2);
						}else interdit(7);}

					else {	//si la chaine ne correspond a rien de connu, on affiche une erreur
						interdit(5);}
				}
			}
		}
	}
	return 0;
}

