/*******************************************************************************\
 *			SeaBattle - D.Bernaudeau, J. Vehent - 2006 	       *
 *									       *
 *	fonction test: contrles d'intgrits de placement et de jeu, tests    *
 *			d'tats "bateau coul" et "fin de partie"	       *
 *									       *
\*******************************************************************************/ 

/*! \file test.c
    \brief fonctions test

    contrles d'intgrits de placement et de jeu, tests d'tats "bateau coul" et "fin de partie"

    \Author D.Bernaudeau
    \Author J. Vehent
 */

//! Detecter le chevauchement entre les bateaux
/*!
\param[in] matrice d'tat des cases
\param[in] taille taille du bateau (=type)
\param[in] x coordonne de la case
\param[in] y coordonne de la case
\param[in] orientation 0:horizontal 2:vertical
\Return -1 s'il y a collision, 0 dans le cas contraire
*/
int set_collision(int matrice[10][10], int taille, int x,int y, int orientation){

	int i,j;

	/// Parcourir l'emplacement dsire pour verifier la presence d'un bateau

	if(orientation==0){
		//Bateau horizontal
		j=y;
		for(i=x;i<=taille+x;i++){
			if(matrice[j][i]!=0) return -1;
		}
	}else{
		//Bateau vertical
		i=x;
		for(j=y;j<=taille+x;j++){
			if(matrice[j][i]!=0) return -1;
		}
	}

	return 0;
}


//! Detecter si le bateau sort de la grille
/*!
\param[in] taille taille du bateau (=type)
\param[in] x coordonne de la case
\param[in] y coordonne de la case
\param[in] orientation 0:horizontal 2:vertical
\Return -1 si le bateau sort de la grille, 0 dans le cas contraire
*/
int sortie_matrice(int taille, int x,int y, int orientation){

	/// La case de dpart + la longueur du bateau ne doit pas etre supprieur  la taille de la grille
	if(orientation==0){
		//Bateau horizontal
		if( (taille+x)>10 ) return -1;
	}else{
		//Bateau vertical
		if( (taille+y)>10 ) return -1;
	}

	return 0;
}

//! Vrifier si un bateau est coule et en modifie son tat
/*!
\param[in] matrice d'tat des cases
\param[in] ligne coordonne y de la case
\param[in] colonne coordonne x de la case
\return 1 si le bateau est coul, 0 dans le cas contraire
*/
int couler(int matrice[10][10], int ligne, int colonne){

		//id_bateau est forcment une valeur ngative
	int i, id_bateau = matrice[ligne][colonne];

	/// Parcourir la ligne  la recherche d'un bateau identique (meme id mais encore positif)
	for(i=0;i<10;i++){
		if(matrice[ligne][i] == (id_bateau*(-1)) )
			/// Si une telle case existe alors le bateau n'est pas coul
			return 0;
	}

	/// Parcourir la colonne  la recherche d'un bateau identique
	for(i=0;i<10;i++){
		if(matrice[i][colonne]== (id_bateau*(-1)) )
			/// Si une telle case existe alors le bateau n'est pas coul
			return 0;
	}
	
	//le bateau est coul, on renvoie 1
	return 1;
}

//! Verifier si la partie est perdu
/*!
\param[in] matrice d'tat des cases
\return 1 si la partie est perdue, 0 si la partie n'est pas terminee
*/
int perdu(int matrice[10][10]){

	int i,j,cpt=0;

	/// Compter le nombre de bateaux couls
	for(i=0;i<10;i++){
		for(j=0;j<10;j++){
			if(matrice[i][j] < -13)
				/// le bateau est coul, on incrmente
				cpt++;
		}
	}

	/// Si le total de cases de bateaux couls est gal  18,
	/// alors tous les bateaux du joueurs sont couls
	if (cpt == 18) return 1;

	///sinon, la partie continue
	else return 0;
}
